<?php

namespace Drupal\a11n_code_form\Form;

use Drupal\authorization_code\AccountIdentifierManager;
use Drupal\authorization_code\AuthorizationCodeRepository;
use Drupal\authorization_code\AuthorizationMethodManager;
use Drupal\authorization_code\Exception\SendCodeException;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A replacement form to Drupal's UserLoginForm.
 */
class QuickLoginForm extends FormBase {

  /**
   * @var \Drupal\authorization_code\AuthorizationMethodManager
   */
  protected $authorizationMethodManager;

  /**
   * @var \Drupal\authorization_code\AccountIdentifierManager
   */
  protected $accountIdentifierManager;

  /**
   * @var \Drupal\authorization_code\AuthorizationCodeRepository
   */
  protected $repository;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'a11n_code_login';
  }

  /**
   * Constructs the QuickLoginForm object.
   *
   * @param \Drupal\authorization_code\AuthorizationMethodManager $authorization_method_manager
   *   The authorization method manager.
   * @param \Drupal\authorization_code\AccountIdentifierManager $account_identifier_manager
   *   The account identifier manager.
   * @param \Drupal\authorization_code\AuthorizationCodeRepository $repository
   *   The authorization code repository.
   */
  public function __construct(AuthorizationMethodManager $authorization_method_manager, AccountIdentifierManager $account_identifier_manager, AuthorizationCodeRepository $repository) {
    $this->authorizationMethodManager = $authorization_method_manager;
    $this->accountIdentifierManager = $account_identifier_manager;
    $this->repository = $repository;
    $manager = \Drupal::service('plugin.manager.account_identifier');
    $manager->getDefinitions();

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.authorization_method'),
      $container->get('plugin.manager.account_identifier'),
      $container->get('authorization_code.repository')
    );
  }

  /**
   * Initializes the data of the form_state object.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function initializeFormState(array &$form, FormStateInterface $form_state) {
    $form_state->set('step', 'identifier');
    $form_state->set('values', []);
    $form_state->set('is_initialized', TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    // Initialize the multi-step form.
    if (empty($form_state->get('is_initialized'))) {
      $this->initializeFormState($form, $form_state);
    }

    $form['actions'] = ['#type' => 'actions'];

    switch ($form_state->get('step')) {
      case 'identifier':
        $this->buildIdentifierForm($form, $form_state);
        break;
      case 'authorize':
        $this->buildAuthorizeForm($form, $form_state);
        break;
      default:
        $form_state->setErrorByName('', t('An error occurred while building the form. If this error persists please contact the site administrator.'));
    }

    return $form;
  }

  /**
   * Add the id of this method to the all of the identifiers in the
   * $supporting_methods array.
   *
   * @param $method_id
   *   The method plugin id.
   * @param $method_definition
   *   The method plugin definition.
   * @param array $supporting_methods
   *   The array of supporting methods to populate.
   * @param bool $has_supported_identifiers
   *   A flag that indicates if a supported identifier was found in this
   *   method definition.
   */
  protected function populateSupportedIdentifiers($method_id, $method_definition, array &$supporting_methods, &$has_supported_identifiers = FALSE) {
    foreach ($method_definition['identifiers'] as $identifier_type) {
      if (isset($supporting_methods[$identifier_type])) {
        $supporting_methods[$identifier_type][] = $method_id;
        $has_supported_identifiers = TRUE;
      }
    }
  }

  /**
   * Builds the form for the identifier step.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  protected function buildIdentifierForm(array &$form, FormStateInterface $form_state) {
    $method_definitions = $this->authorizationMethodManager->getDefinitions();
    $identifier_definitions = $this->accountIdentifierManager->getDefinitions();

    $supporting_methods = array_fill_keys(array_keys($identifier_definitions), []);

    if (count($method_definitions) === 1) {
      $method_definition = reset($method_definitions);
      $method_id = key($method_definitions);

      // Add the id of this method to the all of the identifiers in the
      // $supporting_methods array.
      $this->populateSupportedIdentifiers($method_id, $method_definition, $supporting_methods, $has_supported_identifiers);

      // If the only active method has no supported identifiers, stop building
      // the form and notify the user about the problem.
      if ($has_supported_identifiers === FALSE) {
        $this->messenger()->addError(t('There is an issue with this form. If the problem persists, please contact the site admin.'));
        return;
      }

      // Because there is only one active method, set it as a value element.
      $form['method'] = [
        '#type' => 'hidden',
        '#value' => key($method_definitions)
      ];
    }
    elseif (count($method_definitions) > 1) {
      $options = [];
      foreach ($method_definitions as $method_id => $method_definition) {
        // Add the id of this method to the all of the identifiers in the
        // $supporting_methods array.
        $this->populateSupportedIdentifiers($method_id, $method_definition, $supporting_methods, $has_supported_identifiers);

        // Only add the method if at least one of its supported identifiers is
        // available.
        if ($has_supported_identifiers) {
          $options[$method_id] = $method_definition['label'];
        }
      }

      $form['method'] = [
        '#type' => 'select',
        '#title' => t('Authorization method'),
        '#options' => $options,
        '#required' => TRUE,
        '#empty_option' => t(' - Choose a method - '),
      ];
    }
    else {
      $this->messenger()->addError(t('There is an issue with this form. If the problem persists, please contact the site admin.'));
      return;
    }

    $supporting_methods = array_filter($supporting_methods);

    if (count($supporting_methods) === 1) {
      $identifier_type = key($supporting_methods);
      $identifier_definition = $identifier_definitions[$identifier_type];
      $form['identifier_type'][$identifier_type] = [
        '#type' => 'value',
        '#value' => $identifier_type,
        '#required' => TRUE,
      ];
      $form['identifier'][$identifier_type] = [
        '#type' => 'textfield',
        '#title' => $identifier_definition['field_name'],
        '#default_value' => NULL,
        '#required' => TRUE,
        '#attributes' => [
          'autocorrect' => 'none',
          'autocapitalize' => 'none',
          'spellcheck' => 'false',
          'autofocus' => 'autofocus',
        ],
      ];
    }
    else {
      $form['identifier_type'] = [
        '#type' => 'radios',
        '#title' => t('Choose your identification method'),
        '#required' => TRUE
      ];
      foreach ($supporting_methods as $identifier_type => $supporting_method_ids) {
        $identifier_definition = $identifier_definitions[$identifier_type];

        $form['identifier_type']['#options'][$identifier_type] = $identifier_definition['field_label'];
        $form['identifier_type'][$identifier_type] = [
          '#attributes' => ['required' => TRUE],
          '#attached' => [
            'library' => ['a11n_code_form/identifier_type']
          ]
        ];

        $form['identifier'][$identifier_type] = [
          '#type' => 'textfield',
          '#title' => $identifier_definition['field_label'],
          '#default_value' => NULL,
          '#attributes' => [
            'autocorrect' => 'none',
            'autocapitalize' => 'none',
            'spellcheck' => 'false',
            'autofocus' => 'autofocus',
          ],
        ];

        foreach ($supporting_method_ids as $supporting_method) {
          $form['identifier_type'][$identifier_type]['#states']['enabled'][] = [
            '[name="method"]' => ['value' => $supporting_method]
          ];

          $form['identifier'][$identifier_type]['#states']['visible'][] = [
            '[name="method"]' => ['value' => $supporting_method],
            ':input[name="identifier_type"]' => ['value' => $identifier_type]
          ];
          $form['identifier'][$identifier_type]['#states']['enabled'][] = [
            '[name="method"]' => ['value' => $supporting_method],
            ':input[name="identifier_type"]' => ['value' => $identifier_type]
          ];
          $form['identifier'][$identifier_type]['#states']['required'][] = [
            '[name="method"]' => ['value' => $supporting_method],
            ':input[name="identifier_type"]' => ['value' => $identifier_type]
          ];
        }
      }
    }

    $form['actions']['login'] = [
      '#type' => 'submit',
      '#value' => $this->t('Log in'),
      '#validate' => ['::setFormData', '::validateIdentifier'],
      '#submit' => ['::sendCode'],
    ];
  }

  /**
   * Builds the form for the authorization step.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  protected function buildAuthorizeForm(array &$form, FormStateInterface $form_state) {

    $form['code'] = [
      '#type' => 'number',
      '#title' => $this->t('Code'),
      '#required' => TRUE,
      '#attributes' => [
        'autofocus' => 'autofocus',
      ],
    ];

    $form['actions']['back'] = [
      '#type' => 'submit',
      '#value' => $this->t('Back'),
      '#validate' => [],
      '#submit' => ['::initializeFormState'],
      '#limit_validation_errors' => []
    ];

    $form['actions']['resend'] = [
      '#type' => 'submit',
      '#value' => $this->t('Resend code'),
      '#validate' => [],
      '#submit' => ['::sendCode'],
      '#limit_validation_errors' => []
    ];

    $form['actions']['authorize'] = [
      '#type' => 'submit',
      '#value' => $this->t('Authorize'),
      '#validate' => ['::setFormData', '::validateCode'],
      '#button_type' => 'primary'
    ];
  }

  /**
   * Merges the current values array with the one from the previous step.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function setFormData(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $old_values = $form_state->get('values') ?: [];

    $values = NestedArray::mergeDeep($old_values, $values);

    $values['identifier'] = NestedArray::getValue($values, [$values['identifier_type']]);

    $form_state->set('values', $values);
  }

  /**
   * Uses the authorization method validateIdentifier method, to validate the
   * identifier provided by the user.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function validateIdentifier(array &$form, FormStateInterface $form_state) {
    $identifier_type = $form_state->get(['values', 'identifier_type']);
    $identifier = $form_state->get(['values', 'identifier']);

    if (empty($identifier_type)) {
      $form_state->setError($form['identifier_type'], $this->t('The identifier type field is required.'));
    }
    elseif ($identifier === '') {
      $form_state->setError($form['identifier'][$identifier_type], $this->t('The %title field is required.', [
        '%title' => $form['identifier'][$identifier_type]['#title']
      ]));
    }

    if ($identifier_type_plugin = $this->accountIdentifierManager->getPlugin($identifier_type)) {
      foreach ($this->accountIdentifierManager->getPlugin($identifier_type)->validate($identifier) as $violation) {
        $form_state->setError($form['identifier'][$identifier_type], $violation->getMessage());
      }
    }
    elseif (!empty($identifier_type)) {
      $form_state->setError($form['identifier_type'], t('Please choose a valid identifier.'));
    }
  }

  /**
   * Validates the recieved code against the one in the authorization code
   * repository (using the code-generator service).
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function validateCode(array &$form, FormStateInterface $form_state) {
    $identifier = $form_state->get(['values', 'identifier']);
    $code = $form_state->get(['values', 'code']);

    if (!$this->getAuthorizationMethod($form_state)
      ->verifyCode($identifier, $code)) {
      $form_state->setErrorByName('code', 'The received code is invalid');
    }
  }

  /**
   * Creates and sends the code via the authorization method plugin.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function sendCode(array &$form, FormStateInterface $form_state) {
    $identifier = $form_state->get(['values', 'identifier']);

    try {
      $this->getAuthorizationMethod($form_state)->sendCode($identifier);
      $form_state->setRebuild(TRUE);
      $form_state->set('step', 'authorize');
    } catch (SendCodeException $e) {
      $this->logger('a11n_code_form')->error('%message', ['%message' => $e->getMessage()]);
      $this->messenger()->addError($this->t('Authorization code was not sent due to a server error. If this problem persists, please contact the site administrator.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if ($user = $this->getAuthorizationMethod($form_state)
      ->loadUser($form_state->get(['values', 'identifier']))) {
      // A destination was set, probably on an exception controller,
      if (!$this->getRequest()->request->has('destination')) {
        $form_state->setRedirect(
          'entity.user.canonical',
          ['user' => $user->id()]
        );
      }
      else {
        $this->getRequest()->query->set('destination', $this->getRequest()->request->get('destination'));
      }

      user_login_finalize($user);
    }
    else {
      // TODO move this message to a configuration object.
      \Drupal::messenger()
        ->addWarning($this->t('You have been validated, but no account matched your credentials.<br>You have been redirected to the account registration form.'));
      $form_state->setRedirect('user.register');
    }

  }

  /**
   * Returns the authorization method object for the current form state.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @return \Drupal\authorization_code\Plugin\AuthorizationMethodInterface
   *   An authorization method object.
   */
  protected function getAuthorizationMethod(FormStateInterface $form_state) {
    $method_id = $form_state->get(['values', 'method']);
    $identifier_type = $form_state->get(['values', 'identifier_type']);
    return $this->authorizationMethodManager->getPlugin($method_id, $identifier_type) ?: NULL;
  }

}
