<?php

namespace Drupal\a11n_code_rest\Plugin\rest\resource;

use Drupal\authorization_code\AuthorizationCodeRepository;
use Drupal\authorization_code\AuthorizationMethodManager;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Provides a resource for database watchdog log entries.
 *
 * @RestResource(
 *   id = "authorization_code:send",
 *   label = @Translation("Send authorization code"),
 *   uri_paths = {
 *     "create" = "/api/authorization-code/send"
 *   }
 * )
 */
class SendCodeResource extends ResourceBase {

  /**
   * @var \Drupal\authorization_code\AuthorizationMethodManager
   */
  protected $manager;

  /**
   * @var \Drupal\authorization_code\AuthorizationCodeRepository
   */
  protected $repository;

  /**
   * SendCodeResource constructor.
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param array $serializer_formats
   * @param \Psr\Log\LoggerInterface $logger
   * @param \Drupal\authorization_code\AuthorizationMethodManager $manager
   *   The authorization method manager.
   * @param \Drupal\authorization_code\AuthorizationCodeRepository $repository
   *   The authorization code repository.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, AuthorizationMethodManager $manager, AuthorizationCodeRepository $repository) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->repository = $repository;
    $this->manager = $manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('plugin.manager.authorization_method'),
      $container->get('authorization_code.repository')
    );
  }

  /**
   * The callback method for this resource.
   *
   * @param array $args
   *   An associative array that must contain the following properties:
   *   - method: string
   *       The authorization method plugin id
   *   - identifier_type: string
   *       The account identifier plugin id
   *   - identifier: mixed
   *       The identifier.
   * @return \Drupal\rest\ResourceResponse
   */
  public function post(array $args) {
    $method_id = $args['method'];
    $identifier_type = $args['identifier_type'];
    $identifier = $args['identifier'];
    if (empty($method = $this->manager->getPlugin($method_id, $identifier_type))) {
      throw new BadRequestHttpException('Invalid method or identifier provided');
    }
    if (!$method->isValidIdentifier($identifier)) {
      throw new BadRequestHttpException('Invalid identifier provided');
    }

    $method->sendCode($identifier);
    return new ResourceResponse('Code sent');
  }

  /**
   * {@inheritdoc}
   */
  protected function getBaseRouteRequirements($method) {
    return array_merge(parent::getBaseRouteRequirements($method), [
      '_csrf_token' => 'TRUE'
    ]);
  }

}
