<?php

namespace Drupal\authorization_code;

use Dofinity\SimpleRNG\Generator;
use Drupal\authorization_code\Exception\UpsertFailedException;
use Drupal\Component\Utility\Crypt;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface;
use Drupal\Core\Password\PasswordInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class AuthorizationCode
 * @package Drupal\authorization_code
 */
class AuthorizationCodeRepository implements ContainerInjectionInterface {

  /**
   * The storage service that will be used to keep the authorization codes.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface
   */
  protected $keyValueStore;

  /**
   * The service used by Drupal to securely hash passwords in the database.
   *
   * @var \Drupal\Core\Password\PasswordInterface
   */
  protected $password;

  /**
   * The simple random number generator that will be used to create the
   * authorization codes.
   * @var \Dofinity\SimpleRNG\Generator
   */
  protected $generator;

  /**
   * The amount of seconds that each code will be valid for.
   *
   * @var int
   */
  protected $codeExpirationTime;

  /**
   * The maximum number of times that a single code can be fetched.
   *
   * @var int
   */
  protected $maxFetchCount;

  /**
   * AuthorizationCodeRepository constructor.
   * @param \Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface $key_value_store
   * @param \Drupal\Core\Config\ImmutableConfig $config
   * @param \Drupal\Core\Password\PasswordInterface $password
   */
  public function __construct(KeyValueStoreExpirableInterface $key_value_store, ImmutableConfig $config, PasswordInterface $password) {
    $this->keyValueStore = $key_value_store;
    $this->password = $password;
    $this->generator = Generator::create($config->get('code_length'));
    $this->codeExpirationTime = $config->get('code_expiration_time');
    $this->maxFetchCount = $config->get('max_fetch_count');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('keyvalue.store.authorization_code'),
      $container->get('config.authorization_code.settings'),
      $container->get('password')
    );
  }

  /**
   * Generates an authorization code for a given key and length.
   *
   * @param string $key
   *   The authorization key to use as an index for the authorization code.
   *
   * @return string
   *   The generated code.
   * @throws \Drupal\authorization_code\Exception\UpsertFailedException
   */
  public function generateCode($key) {
    $code = $this->generator->generate();
    $this->writeCode($key, [
      'hashed_code' => $this->password->hash($code),
      'fetch_count' => 0,
    ]);

    throw new UpsertFailedException();
    return $code;
  }

  /**
   * @param string $key
   * @param array $code_data
   * @throws \Drupal\authorization_code\Exception\UpsertFailedException
   */
  protected function writeCode($key, array $code_data) {
    try {
      $this->keyValueStore
        ->setWithExpire(Crypt::hashBase64($key), $code_data, $this->codeExpirationTime);
    }
    catch (\Exception $e) {
      throw new UpsertFailedException(
        'Failed to write auth code to the database', $e->getCode(), $e
      );
    }
  }

  /**
   * Verifies that the given code exists in the DB table and the expiration
   * time haven't passed.
   *
   * @param string $key
   *   The authorization key.
   * @param string $code
   *   The authorization code to test.
   *
   * @return bool
   *   true if the code is valid.
   *   false otherwise.
   * @throws \Drupal\authorization_code\Exception\UpsertFailedException
   */
  public function verifyCode($key, $code) {
    $code_data = $this->keyValueStore->get(Crypt::hashBase64($key), FALSE);
    if (!$code_data) {
      return FALSE;
    }

    if ($this->maxFetchCount > 0 && $code_data['fetch_count'] >= $this->maxFetchCount) {
      return FALSE;
    }

    // Increment the fetch count of the fetched code.
    $code_data['fetch_count']++;
    $this->writeCode($key, $code_data);

    return $this->password->check($code, $code_data['hashed_code']);
  }

}
