<?php

namespace Drupal\authorization_code;

use Drupal\authorization_code\Annotation\AuthorizationMethod;
use Drupal\authorization_code\Plugin\AuthorizationMethodInterface;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;

/**
 * Class AuthorizationMethodManager
 * @package Drupal\authorization_code
 */
class AuthorizationMethodManager extends DefaultPluginManager {

  /**
   * @var \Drupal\authorization_code\Plugin\AuthorizationMethodInterface[][]
   */
  protected $plugins = [];

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $settings;

  /**
   * AuthorizationMethodManager constructor.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Config\ImmutableConfig $settings
   *   The configuration object of the authorization_code module.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler, ImmutableConfig $settings) {
    parent::__construct('Plugin/AuthorizationMethod', $namespaces, $module_handler, AuthorizationMethodInterface::class, AuthorizationMethod::class);
    $this->setCacheBackend($cache_backend, 'authorization_method_plugins');
    $this->alterInfo('authorization_method');
    $this->settings = $settings;
  }

  /**
   * @param string $method_id
   * @param string $account_identifier_type
   * @return \Drupal\authorization_code\Plugin\AuthorizationMethodInterface|false
   *   A fully configured plugin instance. If no instance can be retrieved, FALSE
   *   will be returned.
   */
  public function getPlugin($method_id, $account_identifier_type) {
    return $this->getInstance([
      'id' => $method_id,
      'identifier_type' => $account_identifier_type
    ]);
  }

  /**
   * @param array $options
   * @return false|\Drupal\authorization_code\Plugin\AuthorizationMethodInterface
   *   A fully configured plugin instance. If no instance can be retrieved, FALSE
   *   will be returned.
   */
  public function getInstance(array $options) {
    if (isset($options['id'])) {
      $method_id = $options['id'];
      $identifier_type = $options['identifier_type'];

      if (empty($this->plugins[$method_id][$identifier_type])) {
        try {
          $this->plugins[$method_id][$identifier_type] = $this->createInstance($method_id, $options);
        }
        catch (PluginNotFoundException $e) {
          return FALSE;
        }
      }

      return $this->plugins[$method_id][$identifier_type];
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   *
   * @return \Drupal\authorization_code\Plugin\AuthorizationMethodInterface
   *   A fully configured plugin instance.
   */
  public function createInstance($plugin_id, array $configuration = []) {
    return parent::createInstance($plugin_id, $configuration);
  }

  /**
   * {@inheritdoc}
   */
  protected function alterDefinitions(&$definitions) {
    $definitions = array_filter($definitions, function ($definition) {
      return !$definition['debug'] || $this->settings->get('debug_mode_enabled');
    });
    parent::alterDefinitions($definitions);
  }

}
