<?php

namespace Drupal\authorization_code\Plugin;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\ConstraintViolation;
use Symfony\Component\Validator\ConstraintViolationList;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;

/**
 * Defines the recommended base class for all block plugins.
 */
abstract class AccountIdentifierBase extends PluginBase implements AccountIdentifierInterface, ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Core\TypedData\TypedDataManagerInterface
   */
  protected $typedDataManager;

  /**
   * @var \Drupal\user\UserStorageInterface
   */
  protected $userStorage;

  /**
   * AuthorizationMethodBase constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\TypedData\TypedDataManagerInterface $typed_data_manager
   *   The typed data manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TypedDataManagerInterface $typed_data_manager, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->typedDataManager = $typed_data_manager;
    $this->userStorage = $entity_type_manager->getStorage('user');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('typed_data_manager'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function isValid($identifier) {
    return $this->validate($identifier)->count() == 0;
  }

  /**
   * {@inheritdoc}
   */
  public function validate($identifier) {
    try {
      return $this->createTypedData($identifier)->validate();
    }
    catch (UnexpectedTypeException $e) {
      return new ConstraintViolationList([
        new ConstraintViolation($e->getMessage(), '', [], $identifier, '', $identifier)
      ]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function loadUser($identifier) {
    if ($user_id = $this->getUserId($identifier)) {
      return $this->userStorage->load($user_id);
    }
    else {
      return NULL;
    }
  }

  /**
   * @param mixed $identifier
   * @return \Drupal\Core\TypedData\TypedDataInterface
   */
  public function createTypedData($identifier) {
    return $this->typedDataManager->create($this->getDataDefinition(), $identifier);
  }

  /**
   * @return \Drupal\Core\TypedData\DataDefinitionInterface
   */
  protected function getDataDefinition() {
    return $this->pluginDefinition['data_definition'];
  }

}
