<?php

namespace Drupal\authorization_code\Plugin\AuthorizationMethod;

use Drupal\authorization_code\AccountIdentifierManager;
use Drupal\authorization_code\AuthorizationCodeRepository;
use Drupal\authorization_code\Plugin\AuthorizationMethodBase;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Uses Drupal mail to send the authorization code to users.
 *
 * @AuthorizationMethod(
 *   id = "drupal_mail",
 *   label = @Translation("Drupal mail"),
 *   argument_type = "email",
 *   identifiers = {"unique_user_field:uid", "unique_user_field:name", "unique_user_field:mail"}
 * )
 */
class MailAuthorization extends AuthorizationMethodBase {

  /**
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;

  /**
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  private $logger;

  /**
   * MailAuthorization constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   * @param \Drupal\Core\TypedData\TypedDataManagerInterface $typed_data_manager
   *   The typed data manager.
   * @param \Drupal\authorization_code\AuthorizationCodeRepository $repository
   *   The authorization code repository.
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\authorization_code\AccountIdentifierManager $account_identifier_manager
   *   The account identifier plugin manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerChannelInterface $logger, TypedDataManagerInterface $typed_data_manager, AuthorizationCodeRepository $repository, MailManagerInterface $mail_manager, LanguageManagerInterface $language_manager, AccountIdentifierManager $account_identifier_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $typed_data_manager, $repository, $account_identifier_manager);
    $this->mailManager = $mail_manager;
    $this->languageManager = $language_manager;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.channel.authorization_code'),
      $container->get('typed_data_manager'),
      $container->get('authorization_code.repository'),
      $container->get('plugin.manager.mail'),
      $container->get('language_manager'),
      $container->get('plugin.manager.account_identifier')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function doSendCode($argument, $code) {
    if ($user = $this->accountIdentifier->loadUser($argument)) {
      $this->mailManager->mail(
        'system',
        'authorization_code',
        $user->getEmail(),
        $this->getPreferredLangcode($argument),
        $this->buildMailParameters($argument, $code)
      );
    }
  }

  /**
   * {@inheritdoc}
   */
  public function loadUser($argument) {
    return user_load_by_mail($argument) ?: NULL;
  }

  /**
   * Builds the params array as expected by the system implementation of
   * hook_mail.
   *
   * @param string $mail
   *   An email address.
   * @param string $code
   *   An authorization code string.
   *
   * @return array
   *   The parameters array for the mail function.
   *
   * @see system_mail()
   */
  protected function buildMailParameters($mail, $code) {
    $site_name = \Drupal::config('system.site')->get('name');
    $args = ['@code' => $code, '@site' => $site_name];
    $options = ['langcode' => $this->getPreferredLangcode($mail)];

    // TODO replace message & subject with configured variables.
    $subject_template = '@code is your @site account login code';
    $message_template = '<p>' . implode('</p><p>', [
        'Hi,',
        'We received your request log into your @site account.',
        'Enter the following code: @code'
      ]) . '</p>';

    $params['context'] = [
      'subject' => $this->t($subject_template, $args, $options),
      'message' => $this->t($message_template, $args, $options),
    ];
    return $params;
  }

  /**
   * Returns the preferred language code per received mail parameter.
   * If there exists a user with this email address, then return the language
   * code that they set as preferred.
   * Otherwise, use the current language code.
   *
   * @param string $mail
   *   An email address.
   * @return string
   *   A language code.
   */
  protected function getPreferredLangcode($mail) {
    /** @var \Drupal\user\UserInterface $user */
    if ($user = user_load_by_mail($mail)) {
      return $user->getPreferredLangcode();
    }
    else {
      return $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)
        ->getId();
    }
  }

}
