<?php

namespace Drupal\authorization_code\Plugin;

use Drupal\authorization_code\AccountIdentifierManager;
use Drupal\authorization_code\AuthorizationCodeRepository;
use Drupal\authorization_code\Exception\SendCodeException;
use Drupal\authorization_code\Exception\UpsertFailedException;
use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the recommended base class for all block plugins.
 */
abstract class AuthorizationMethodBase extends PluginBase implements AuthorizationMethodInterface, ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Core\TypedData\TypedDataManagerInterface
   */
  protected $typedDataManager;

  /**
   * @var \Drupal\authorization_code\AuthorizationCodeRepository
   */
  protected $repository;

  /**
   * @var \Drupal\authorization_code\Plugin\AccountIdentifierInterface
   */
  protected $accountIdentifier;

  /**
   * AuthorizationMethodBase constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\TypedData\TypedDataManagerInterface $typed_data_manager
   *   The typed data manager.
   * @param \Drupal\authorization_code\AuthorizationCodeRepository $repository
   *   The authorization code repository.
   * @param \Drupal\authorization_code\AccountIdentifierManager $account_identifier_manager
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, TypedDataManagerInterface $typed_data_manager, AuthorizationCodeRepository $repository, AccountIdentifierManager $account_identifier_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    if (!isset($configuration['identifier_type'])) {
      throw new PluginException('identifier_type is missing from the configuration.');
    }
    if (!in_array($configuration['identifier_type'], $plugin_definition['identifiers'])) {
      throw new PluginException('The received account identifier plugin id is not supported by this authorization method plugin.');
    }

    $this->typedDataManager = $typed_data_manager;
    $this->repository = $repository;
    $this->accountIdentifier = $account_identifier_manager->getPlugin($configuration['identifier_type']);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('typed_data_manager'),
      $container->get('authorization_code.repository'),
      $container->get('plugin.manager.account_identifier')
    );
  }

  /**
   * Concatenates this plugin id with a given account identifier to create a key
   * that will be associated with the generated code.
   *
   * @param string $identifier
   *   The account identification string.
   * @return string
   *   The string that will be associated with our generated code.
   */
  protected function getKey($identifier) {
    return implode(':', [$this->getPluginId(), $identifier]);
  }

  /**
   * {@inheritdoc}
   */
  public function sendCode($identifier) {
    $key = $this->getKey($identifier);
    try {
      $this->doSendCode($identifier, $this->repository->generateCode($key));
    } catch (UpsertFailedException $e) {
      throw SendCodeException::fromUpsertFailedException($e);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function verifyCode($identifier, $code) {
    $key = $this->getKey($identifier);
    return $this->repository->verifyCode($key, $code);
  }

  /**
   * This method is to be implemented by the actual authorization method plugin
   * and will be used to send to code to the user.
   *
   * @param mixed $identifier
   *   Authorization method identifier.
   * @param string $code
   *   The generated code.
   */
  abstract protected function doSendCode($identifier, $code);

  /**
   * {@inheritdoc}
   */
  public function isValidIdentifier($identifier) {
    return $this->accountIdentifier->isValid($identifier);
  }

  /**
   * {@inheritdoc}
   */
  public function validateIdentifier($identifier) {
    return $this->accountIdentifier->validate($identifier);
  }

}
