<?php

namespace Drupal\authorization_code\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\Validation\ConstraintManager;
use Drupal\Core\Validation\Plugin\Validation\Constraint\UniqueFieldConstraint;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The Deriver class for the unique_user_field AccountIdentifier plugin.
 */
class UniqueUserField extends DeriverBase implements ContainerDeriverInterface {

  /**
   * @var \Drupal\Core\Validation\ConstraintManager
   */
  protected $constraintManager;

  /**
   * @var \Drupal\Core\Field\FieldDefinitionInterface[]
   */
  protected $fieldDefinitions;

  /**
   * UniqueUserField constructor.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Validation\ConstraintManager $constraint_manager
   *   The validation constraint manager.
   */
  public function __construct(EntityFieldManagerInterface $field_manager, ConstraintManager $constraint_manager) {
    $this->fieldDefinitions = $field_manager->getFieldDefinitions('user', 'user');
    $this->constraintManager = $constraint_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_field.manager'),
      $container->get('validation.constraint')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    foreach ($this->fieldDefinitions as $field_name => $field_definition) {
      if ($main_property = $this->getMainPropertyDefinition($field_definition)) {
        list($main_property_name, $main_property_definition) = $main_property;
        if ($this->hasUniqueFieldConstraint($field_definition)) {
          $this->derivatives[$field_name] = $base_plugin_definition;
          $this->derivatives[$field_name]['data_definition'] = $main_property_definition;
          $this->derivatives[$field_name]['field_name'] = $field_name;
          $this->derivatives[$field_name]['field_label'] = $field_definition->getLabel();
          $this->derivatives[$field_name]['main_property_name'] = $main_property_name;
        }
      }
    }

    return parent::getDerivativeDefinitions($base_plugin_definition);
  }

  /**
   * Checks if one of the entity field constraint is an instance of the
   * UniqueField constraint.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition object.
   * @return bool
   *   True if an instance of UniqueField constraint was found, false otherwise.
   *
   * @see FieldDefinitionInterface::getConstraints()
   * @see UniqueFieldConstraint
   */
  protected function hasUniqueFieldConstraint(FieldDefinitionInterface $field_definition) {
    if ($field_definition->getName() === 'uid') {
      return TRUE;
    }
    foreach ($field_definition->getConstraints() as $constraint_name => $constraint_definition) {
      $constraint = $this->constraintManager->create($constraint_name, $constraint_definition);
      if ($constraint instanceof UniqueFieldConstraint) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Returns the main property of the field, if any.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition object.
   *
   * @return array|null
   *   An array containing the property name and definition, or NULL if there is none.
   *
   * @see \Drupal\Core\Field\TypedData\FieldItemDataDefinitionInterface::getMainPropertyName()
   * @see \Drupal\Core\Field\TypedData\FieldItemDataDefinitionInterface::getPropertyDefinition()
   */
  protected function getMainPropertyDefinition(FieldDefinitionInterface $field_definition) {
    /** @var \Drupal\Core\Field\TypedData\FieldItemDataDefinitionInterface $item_definition */
    $item_definition = $field_definition->getItemDefinition();
    if ($main_property_name = $item_definition->getMainPropertyName()) {
      return [
        $main_property_name,
        $item_definition->getPropertyDefinition($main_property_name)
      ];
    }

    return NULL;
  }

}
