<?php

namespace Drupal\Tests\authorization_code\Kernel\Plugin\AccountIdentifier;

use Drupal\authorization_code\AccountIdentifierManager;
use Drupal\authorization_code\Plugin\AccountIdentifierInterface;
use Drupal\authorization_code\Plugin\AuthorizationMethodInterface;
use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Tests\authorization_code\Kernel\Plugin\PluginKernelTestBase;
use Drupal\user\UserInterface;
use Symfony\Component\Validator\ConstraintViolation;
use Symfony\Component\Validator\ConstraintViolationList;

/**
 * Confirm that the drupal_mail AuthorizationMethod plugin works properly
 *
 * @group authorization_code
 */
class MailAuthorization extends PluginKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $pluginInterface = AuthorizationMethodInterface::class;

  /**
   * {@inheritdoc}
   */
  protected static $pluginName = 'authorization_method';

  /**
   * @var \Drupal\authorization_code\AuthorizationMethodManager
   */
  protected $manager;

  protected $mockManager;

  protected $mockPlugin;

  protected $maxFetchCount;

  protected $capturedMail;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $this->config('system.site')->set('name', 'site')->save(TRUE);
    $this->installSchema('system', ['key_value_expire']);
    $this->maxFetchCount = $this->config('authorization_code.settings')
      ->get('max_fetch_count');
  }

  /**
   * @param \Drupal\user\UserInterface $user
   *
   * @return \PHPUnit_Framework_MockObject_MockObject
   */
  protected function createMockAccountIdentifierManager(UserInterface $user) {
    $this->mockManager = $this->getMockBuilder(AccountIdentifierManager::class)
      ->disableOriginalConstructor()
      ->getMock();
    $this->mockManager->expects($this->any())
      ->method('getPlugin')
      ->withAnyParameters()
      ->willReturn($this->createMockAccountIdentifier($user));
    return $this->mockManager;
  }

  /**
   * @param \Drupal\user\UserInterface $user
   *
   * @return \PHPUnit_Framework_MockObject_MockObject
   */
  protected function createMockAccountIdentifier(UserInterface $user) {
    $this->mockPlugin = $this->getMockBuilder(AccountIdentifierInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    $this->mockPlugin->expects($this->any())
      ->method('isValid')->with('valid_identifier')->willReturn(TRUE);
    $this->mockPlugin->expects($this->any())
      ->method('isValid')->with('invalid_identifier')->willReturn(FALSE);

    $this->mockPlugin->expects($this->any())
      ->method('validate')
      ->with('valid_identifier')
      ->willReturn(new ConstraintViolationList());

    $this->mockPlugin->expects($this->any())
      ->method('validate')
      ->with('invalid_identifier')
      ->willReturn(new ConstraintViolationList([
        new ConstraintViolation('invalid identifier', 'invalid identifier', [], '', '', 'invalid_identifier'),
      ]));

    $this->mockPlugin->expects($this->any())
      ->method('getUserId')
      ->with($this->logicalOr(
        $this->equalTo('valid_identifier'),
        $this->equalTo('invalid_identifier')
      ))
      ->willReturnCallback(function ($identifier) use ($user) {
        return $identifier === 'valid_identifier' ? $user->id() : NULL;
      });

    $this->mockPlugin->expects($this->any())
      ->method('loadUser')
      ->with($this->logicalOr(
        $this->equalTo('valid_identifier'),
        $this->equalTo('invalid_identifier')
      ))
      ->willReturnCallback(function ($identifier) use ($user) {
        return $identifier === 'valid_identifier' ? $user : NULL;
      });

    return $this->mockPlugin;
  }

  /**
   * @param $identifier
   *
   * @return \PHPUnit_Framework_MockObject_MockObject
   */
  public function createMockMailManager($identifier) {
    $mock = $this->getMockBuilder(MailManagerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    $mock->expects($this->any())
      ->method('mail')
      ->withAnyParameters()
      ->willReturnCallback([$this, 'mockMail']);

    return $mock;
  }

  /**
   * @param $module
   * @param $key
   * @param $to
   * @param $langcode
   * @param array $params
   */
  public function mockMail($module, $key, $to, $langcode, $params = []) {
    $this->capturedMail = [
      'module' => $module,
      'key' => $key,
      'to' => $to,
      'langcode' => $langcode,
      'params' => $params,
    ];
  }

  /**
   * @param string $plugin_id
   * @param string $identifier_type
   * @param string $identifier
   * @param array $user_data
   *
   * @dataProvider sendCodeProvider
   */
  public function testCode($plugin_id, $identifier_type, $identifier, $user_data) {
    $user = $this->createUser($user_data);
    $this->createDummyUsers($user->id() + 1);

    $this->container->set('plugin.manager.mail', $this->createMockMailManager($identifier));
    $this->container->set('plugin.manager.account_identifier', $this->createMockAccountIdentifierManager($user));

    $plugin = $this->manager->getPlugin($plugin_id, $identifier_type);
    $plugin->sendCode($identifier);

    $context = $this->capturedMail['params']['context'];

    $subject_regex = '/^\d+(?= is your site account login code)/';
    $this->assertRegExp($subject_regex, (string) $context['subject'], 'Code sent.');

    preg_match($subject_regex, (string) $context['subject'], $matches);
    $code = $matches[0];

    for ($i = 0; $i < $this->maxFetchCount; $i++) {
      $this->assertTrue($plugin->verifyCode($identifier, $code), 'Code verified #' . $i);
    }

    $this->assertFalse($plugin->verifyCode($identifier, $code), "Code is invalid after {$i} tries");
  }

  /**
   * @param string $plugin_id
   * @param string $identifier_type
   * @param string $identifier
   * @param array $user_data
   *
   * @dataProvider sendCodeInvalidProvider
   */
  public function testInvalidCode($plugin_id, $identifier_type, $identifier, $user_data) {
    $user = $this->createUser($user_data);

    $this->container->set('plugin.manager.mail', $this->createMockMailManager($identifier));
    $this->container->set('plugin.manager.account_identifier', $this->createMockAccountIdentifierManager($user));

    $plugin = $this->manager->getPlugin($plugin_id, $identifier_type);
    $plugin->sendCode($identifier);

    $context = $this->capturedMail['params']['context'];

    $this->assertEmpty((string) $context['subject'], 'No code was sent.');
    $code = 'invalid_code';
    $this->assertFalse($plugin->verifyCode($identifier, $code), "Fake code is invalid");
  }

  /**
   * @return array
   */
  public function sendCodeProvider() {
    return [
      'positive:uid' => [
        'drupal_mail',
        'unique_user_field:uid',
        'valid_identifier',
        [
          'uid' => 2,
          'mail' => 'mail@example',
        ],
      ],
      'positive:mail' => [
        'drupal_mail',
        'unique_user_field:mail',
        'valid_identifier',
        [
          'mail' => 'mail@example',
        ],
      ],
      'positive:name' => [
        'drupal_mail',
        'unique_user_field:mail',
        'valid_identifier',
        [
          'name' => 'Some name',
          'mail' => 'mail@example',
        ],
      ],
    ];
  }

  /**
   * @return array
   */
  public function sendCodeInvalidProvider() {
    return [
      'negative:uid' => [
        'drupal_mail',
        'unique_user_field:uid',
        'invalid_identifier',
        [
          'uid' => 2,
          'mail' => 'mail@example',
        ],
      ],
      'negative:mail' => [
        'drupal_mail',
        'unique_user_field:mail',
        'invalid_identifier',
        [
          'mail' => 'mail@example',
        ],
      ],
      'negative:name' => [
        'drupal_mail',
        'unique_user_field:mail',
        'invalid_identifier',
        [
          'name' => 'Some name',
          'mail' => 'mail@example',
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function pluginNamesProvider() {
    return [
      'positive' => [
        'drupal_mail',
        [
          'identifier_type' => 'unique_user_field:uid',
        ],
        TRUE,
      ],
      'negative:missingIdentifierType' => [
        'drupal_mail',
        [],
        PluginException::class,
        'identifier_type is missing from the configuration.',
      ],
      'negative:invalidIdentifierType' => [
        'drupal_mail',
        [
          'identifier_type' => 'invalid_identifier',
        ],
        PluginException::class,
        'The received account identifier plugin id is not supported by this authorization method plugin.',
      ],
      'negative:fakePluginId' => ['not_drupal_mail', []],
    ];
  }

}
